% CLASS OrthMesh, meshing d-hypercubes with simplices or hypercubes.
%
% SEE
%   OrthMesh/OrthMesh : constructor
%   OrthMesh/plot     : plot function
%   OrthMesh/disp     : display information on an OrthMesh object
%
%    Copyright (C) 2016- F. Cuvelier 
classdef OrthMesh %< handle 

  properties %(SetAccess = protected)
    d = 0; % space dimension
    Mesh=[]; % main mesh (EltMesh)
    Faces={}; % cells of m-face meshes
    type=[]; % 'simplicial' or 'orthotope'
    box=[]; 
  end
  
  methods
    function self = OrthMesh(d,N,varargin)
    % OrthMesh constructor
    %   
    % USAGE
    %   Oh=OrthMesh(d,N, ...)
    %
    %   Parameter d is the space dimension and parameter N the number of
    %   iscretization in each direction. N is either a 1-by-d array
    %   or a scalar.
    %   The d,N parameters can be followed by key/value pairs to specify
    %   options. The keys will be the strings:
    %     'type' to specify the kind of mesh with value equal to 
    %            'simplicial' (default) or 'orthotope'
    %     'box' to describe the orthotope to mesh. The default orthotope is
    %           the unit d-hypercube [0,1]^d. The associated value is 
    %           an d-by-2 array.
    %
    % SAMPLES
    %   Oh=OrthMesh(3,10);
    %   Oh=OrthMesh(2,[5,7],'type','orthotope');
    %   Oh=OrthMesh(3,[5,7,6],'box',[-2,2;-1,1;0,1]);
    %
    %    Copyright (C) 2016- F. Cuvelier 
      if length(N)==1
        N=N*ones(1,d);
      else
        assert(size(N,2)==d)
      end
      p = inputParser;
      p.addParamValue('type', 'simplicial', @(x) ismember(x,{'simplicial','orthotope'}) );
      p.addParamValue('box', [zeros(d,1),ones(d,1)], @(x) (sum(size(x)==[d,2])==2) );
      p.addParamValue('m_min', 0, @(x) (x<=d) && (x>=0) );
      p.addParamValue('mapping', []);
      p.parse(varargin{:});
      R=p.Results;
      self.box=R.box;
      self.type=R.type;
      if strcmp(R.type,'simplicial')
        funMesh=@(N) CartesianGrid.Triangulation(N);
        funFaces=@(N,m) CartesianGrid.genLevelFaces(N,m,'type','simplicial');
      else
        funMesh=@(N) CartesianGrid.TessHyp(N);
        funFaces=@(N,m) CartesianGrid.genLevelFaces(N,m,'type','orthotope');
      end
      self.d=d;
      mappingfun=@(q) diag(1./N)*q;% mapping to unit hypercube
      mappingfun=@(q) fc_hypermesh.MappingBox(mappingfun(q),R.box);
      [q,me]=funMesh(N);
      if ~isempty(R.mapping)
        mappingfun=@(q) R.mapping(mappingfun(q));
      end
      labName=fc_hypermesh.funLabName(d,d);
      self.Mesh=EltMesh(d,d,mappingfun(q),me,[],'label',labName(1));
      k=1;
      for m=d-1:-1:R.m_min
        labName=fc_hypermesh.funLabName(d,m);
        sTh=funFaces(N,m);
        nsTh=length(sTh);
        colors = fc_tools.graphics.selectColors(nsTh);
          for j=1:length(sTh)
            sTh(j).set('color',colors(j,:),'label',labName(j),'mapping',mappingfun);
%           Use set property due to 'BUG' in Octave?            
%            sTh(j).mapping(mappingfun);
%            sTh(j).color=colors(j,:);
%            sTh(j).label=labName(j);
          end
        self.Faces{k}=sTh;
        k=k+1;
      end
    end
  end
end